package com.muyu.server.controller;

import cn.hutool.core.bean.BeanUtil;
import com.muyu.cloud.common.many.datasource.constents.DatasourceContent;
import com.muyu.common.core.constant.SecurityConstants;
import com.muyu.common.core.domain.Result;
import com.muyu.common.domain.Enterprise;
import com.muyu.common.system.domain.Business;
import com.muyu.common.system.domain.SysUser;
import com.muyu.common.system.remote.RemoteUserService;
import com.muyu.common.util.PageUtils;
import com.muyu.server.controller.form.DeleteEnterpriseByIds;
import com.muyu.server.controller.form.InsertEnterprise;
import com.muyu.server.controller.form.SearchEnterpriseName;
import com.muyu.server.controller.form.UpdateEnterprise;
import com.muyu.server.service.EnterpriseService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.support.EncodedResource;
import org.springframework.jdbc.datasource.init.ScriptUtils;
import org.springframework.web.bind.annotation.*;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 企业运营控制层
 * @author yuping
 * @Description 企业运营控制层
 * @Version 1.0
 * @Data 2024-09-26 20:24:05
 */
@RestController
@RequestMapping("/enterprise")
@Log4j2
@Tag(name = "企业运营管理",description = "企业运营管理")
public class EnterpriseController {

    @Autowired
    private EnterpriseService enterpriseService;

    @Autowired
    private RemoteUserService remoteUserService;


    /**
     * 查询分页企业信息
     * @param form
     * @return
     */
    @PostMapping("/selectEnterprise")
    @Operation(summary = "查询分页企业信息",description = "查询分页企业信息")
    public Result selectEnterprise(@RequestBody @Valid SearchEnterpriseName form) {
        Integer page = form.getPage();
        Integer length = form.getLength();
        int start = (page - 1) * length;
        Map param = BeanUtil.beanToMap(form);
        param.put("start", start);
        PageUtils pageUtils = enterpriseService.selectEnterprise(param);

        System.out.println(pageUtils);

        return Result.success(pageUtils);
    }


    /**
     * 新增企业信息
     * @param form
     * @return
     */
    @PostMapping("/insert")
    @Operation(summary = "新增企业信息",description = "新增企业信息")
    public Result insert(@RequestBody @Valid InsertEnterprise form){
        Enterprise enterprise = new Enterprise();

        enterprise.setEnterpriseName(form.getEnterpriseName());
        enterprise.setEnterpriseCarCount(form.getEnterpriseCarCount());
        enterprise.setEnterpriseFenceCount(form.getEnterpriseFenceCount());

        int rows = enterpriseService.insert(enterprise);
        if (rows == 0){
            return Result.error("新增失败");
        }

        Business business = new Business();
        business.setFirmName(form.getEnterpriseName());
        business.setDatabaseName(form.getEnterpriseDatabaseName());
        remoteUserService.settlementEnterpriseInfo(business, SecurityConstants.INNER);

        SysUser sysUser = new SysUser();
        sysUser.setDatabaseName(form.getEnterpriseDatabaseName());
        sysUser.setUserName(form.getEnterpriseName());
        sysUser.setCreateTime(new Date());
        remoteUserService.addUser(sysUser, SecurityConstants.INNER);
        String createDatabaseUrl="jdbc:mysql://"+ DatasourceContent.IP+":"+DatasourceContent.PORT+"?useUnicode=true&characterEncoding=utf8&zeroDateTimeBehavior=convertToNull&useSSL=true&serverTimezone=GMT%2B8";
        String createDatabaseSql = "CREATE DATABASE IF NOT EXISTS " + form.getEnterpriseDatabaseName() + ";";

        try (Connection adminConn = DriverManager.getConnection(createDatabaseUrl, DatasourceContent.USER_NAME, DatasourceContent.PASSWORD);
             Statement stmt = adminConn.createStatement()) {

            boolean success = stmt.execute(createDatabaseSql);
            if (success) {
                log.info("数据库 {} 创建失败", form.getEnterpriseDatabaseName());

            } else {
                log.warn("数据库 {} 创建成功", form.getEnterpriseDatabaseName());

                // 切换到新的数据库连接
                Connection connection = null;
                try {
                    String url = "jdbc:mysql://47.101.53.251:3306/" + form.getEnterpriseDatabaseName() + "?useUnicode=true&characterEncoding=utf8&zeroDateTimeBehavior=convertToNull&useSSL=true&serverTimezone=GMT%2B8";
                    String user = "root";
                    String pwd = "Lw030106";
                    String driverClassName = "com.mysql.cj.jdbc.Driver";
                    Class.forName(driverClassName);
                    connection = DriverManager.getConnection(url, user, pwd);
                } catch (ClassNotFoundException e) {
                    throw new RuntimeException(e);
                }
                ClassPathResource rc = new ClassPathResource("static/saas.sql");
                EncodedResource er = new EncodedResource(rc, "utf-8");
                ScriptUtils.executeSqlScript(connection, er);


            }

        } catch (SQLException e) {
            e.printStackTrace();
            log.error("连接数据库时发生错误或创建数据库失败", e);
        }
        return Result.success(rows);
    }


    /**
     * 根据编号查询企业信息
     * @param enterpriseId
     * @return
     */
    @GetMapping("/searchById")
    @Operation(summary = "根据编号查询企业信息",description = "根据编号查询企业信息")
    public Result searchById(@RequestParam("enterpriseId") Integer enterpriseId){
        HashMap map = enterpriseService.searchById(enterpriseId);
        return Result.success(map);
    }


    /**
     * 修改企业信息
     * @param form
     * @return
     */
    @PostMapping("/updateEnterEnterprise")
    @Operation(summary = "修改企业信息",description = "修改企业信息")
    public Result updateEnterprise(@RequestBody @Valid UpdateEnterprise form){
        Enterprise enterprise = new Enterprise();

        enterprise.setEnterpriseId(form.getEnterpriseId());
        enterprise.setEnterpriseName(form.getEnterpriseName());
        enterprise.setEnterpriseCarCount(form.getEnterpriseCarCount());
        enterprise.setEnterpriseFenceCount(form.getEnterpriseFenceCount());

        int rows = enterpriseService.updateEnterprise(enterprise);
        return Result.success(rows);
    }


    /**
     * 删除企业信息
     * @param form
     * @return
     */
    @PostMapping("/deleteByIds")
    @Operation(summary = "删除企业信息",description = "删除企业信息")
    public Result deleteByIds(@RequestBody @Valid DeleteEnterpriseByIds form){
        int rows = enterpriseService.deleteByIds(form.getIds());
        return Result.success(rows);
    }


}
